==> svm-20.1.0-sources.jar: com/oracle/svm/core/log/StringBuilderLog.java <==
/*
 * Copyright (c) 2017, 2017, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

package com.oracle.svm.core.log;

import org.graalvm.nativeimage.c.type.CCharPointer;
import org.graalvm.word.UnsignedWord;

import com.oracle.svm.core.annotate.RestrictHeapAccess;
import com.oracle.svm.core.annotate.Uninterruptible;

public class StringBuilderLog extends RealLog {
    private final StringBuilder stringBuilder = new StringBuilder();

    public StringBuilderLog() {
    }

    @RestrictHeapAccess(access = RestrictHeapAccess.Access.UNRESTRICTED, overridesCallers = true, reason = "This implementation allocates.")
    @Uninterruptible(reason = "Called from uninterruptible code.", calleeMustBe = false)
    @Override
    protected Log rawBytes(CCharPointer bytes, UnsignedWord length) {
        for (int i = 0; length.aboveThan(i); i++) {
            char currentChar = (char) bytes.read(i);
            stringBuilder.append(currentChar);
        }
        return this;
    }

    @Override
    public Log flush() {
        // noop
        return this;
    }

    public String getResult() {
        return stringBuilder.toString();
    }
}

==> svm-22.0.0-sources.jar: com/oracle/svm/core/log/StringBuilderLog.java <==
/*
 * Copyright (c) 2017, 2017, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

package com.oracle.svm.core.log;

import org.graalvm.nativeimage.c.type.CCharPointer;
import org.graalvm.word.UnsignedWord;

import com.oracle.svm.core.annotate.RestrictHeapAccess;
import com.oracle.svm.core.annotate.Uninterruptible;

public class StringBuilderLog extends RealLog {
    private final StringBuilder stringBuilder = new StringBuilder();

    public StringBuilderLog() {
    }

    @RestrictHeapAccess(access = RestrictHeapAccess.Access.UNRESTRICTED, reason = "This implementation allocates.")
    @Uninterruptible(reason = "Called from uninterruptible code.", calleeMustBe = false)
    @Override
    protected Log rawBytes(CCharPointer bytes, UnsignedWord length) {
        for (int i = 0; length.aboveThan(i); i++) {
            char currentChar = (char) bytes.read(i);
            stringBuilder.append(currentChar);
        }
        return this;
    }

    @Override
    public Log flush() {
        // noop
        return this;
    }

    public String getResult() {
        return stringBuilder.toString();
    }
}
